module ov5640x2_ddr2_vga(
	Clk50M,
	Clk27M,
	Rst_n,

	Camera1_sclk,
	Camera1_sdat,
	Camera1_vsync,
	Camera1_href,
	Camera1_pclk,
	Camera1_xclk,
	Camera1_data,
	Camera1_rst_n,
	Camera1_pwdn,
	
	Camera2_sclk,
	Camera2_sdat,
	Camera2_vsync,
	Camera2_href,
	Camera2_pclk,
	Camera2_xclk,
	Camera2_data,
	Camera2_rst_n,
	Camera2_pwdn,
	
	DDR2_ADDR,
	DDR2_BA,
	DDR2_CAS,
	DDR2_CKE,
	DDR2_CLK,
	DDR2_CLK_N,
	DDR2_CS,
	DDR2_DM,
	DDR2_DQ,
	DDR2_DQS,
	DDR2_ODT,
	DDR2_RAS,
	DDR2_WE,

	vga_data,
	vga_hs,
	vga_vs,
	vga_clk,
	vga_de
);
	input               	Clk50M;     //系统时钟
	input               	Clk27M;     //系统时钟
	input               	Rst_n;      //系统复位信号
	
	//cmos1 interface
	output					Camera1_sclk;
	inout						Camera1_sdat;
	input						Camera1_vsync;
	input						Camera1_href;
	input						Camera1_pclk;
	output					Camera1_xclk;
	input 	[7:0]			Camera1_data;
	output					Camera1_rst_n;
	output					Camera1_pwdn;
	
	//cmos2 interface
	output					Camera2_sclk;
	inout						Camera2_sdat;
	input						Camera2_vsync;
	input						Camera2_href;
	input						Camera2_pclk;
	output					Camera2_xclk;
	input 	[7:0]			Camera2_data;
	output					Camera2_rst_n;
	output					Camera2_pwdn;

	//ddr2 control
	output	[12:0]		DDR2_ADDR;
	output	[2:0]			DDR2_BA;
	output					DDR2_CAS;
	output					DDR2_CKE;
	inout						DDR2_CLK;
	inout						DDR2_CLK_N;
	output					DDR2_CS;
//	output	[1:0]			DDR2_DM;
//	inout		[15:0]		DDR2_DQ;
//	inout		[1:0]			DDR2_DQS;
	output	[3:0]			DDR2_DM;
	inout		[31:0]		DDR2_DQ;
	inout		[3:0]			DDR2_DQS;
	output					DDR2_ODT;
	output					DDR2_RAS;
	output					DDR2_WE;
	
	//vga output
	output  	[23:0] 		vga_data;
	output       			vga_hs;
	output       			vga_vs;
	output       			vga_clk;
	output       			vga_de;
	
	reg [19:0]cnt;
	reg sys_rst_n;
	always@(posedge Clk50M or negedge Rst_n)
	begin
		if(!Rst_n)
			cnt <= 20'd0;
		else if(cnt < 20'hffff0)
			cnt <= cnt + 8'd1;
		else
			cnt <= cnt;
	end	
	
	always@(posedge Clk50M or negedge Rst_n)
	begin
		if(!Rst_n)
			sys_rst_n <= 1'b0;
		else if(cnt >= 20'hffff0)
			sys_rst_n <= 1'b1;
		else
			sys_rst_n <= 1'b0;
	end
	
	wire GLOBAL_RES;
	wire clk_50m;
	wire clk_24m;
	wire clk_vga;
	assign Camera1_xclk = clk_24m;
	assign Camera2_xclk = clk_24m;
	pll pll(
		.inclk0(Clk27M),
		.c0(clk_50m),
		.c1(clk_24m),
		.c2(clk_vga)
	);
	
	wire clk_30m;
	pll_wr pll_wr_inst(
		.inclk0(Clk50M),
		.c0(clk_30m)
	);

	
	localparam RGB = 0;
	localparam JPEG = 1;
	
	//parameter IMAGE_WIDTH  = 640; //图片宽度1280=640*2
	parameter IMAGE_WIDTH  = 960; //图片宽度 190=960*2
	parameter IMAGE_HEIGHT = 1080; //图片高度(≤720)
	parameter IMAGE_FLIP   = 0;   //0：不翻转，1：上下翻转
	parameter IMAGE_MIRROR = 0;   //0：不镜像，1：左右镜像

	//摄像头1,摄像头2初始化配置
	wire Init_Done1;
	wire Init_Done2;
	wire Init_Done = (Init_Done1 && Init_Done2);
	camera_init
	#(
		.IMAGE_TYPE(RGB),
		.IMAGE_WIDTH(IMAGE_WIDTH),
		.IMAGE_HEIGHT(IMAGE_HEIGHT),
		.IMAGE_FLIP(IMAGE_FLIP),
		.IMAGE_MIRROR(IMAGE_MIRROR)
	)
	camera_init1(
		.Clk(clk_50m),
		.Rst_n(sys_rst_n & GLOBAL_RES),
		.Init_Done(Init_Done1),
		.camera_rst_n(Camera1_rst_n),
		.camera_pwdn(Camera1_pwdn),
		.i2c_sclk(Camera1_sclk),
		.i2c_sdat(Camera1_sdat)
	),
	camera_init2(
		.Clk(clk_50m),
		.Rst_n(sys_rst_n & GLOBAL_RES),
		.Init_Done(Init_Done2),
		.camera_rst_n(Camera2_rst_n),
		.camera_pwdn(Camera2_pwdn),
		.i2c_sclk(Camera2_sclk),
		.i2c_sdat(Camera2_sdat)
	);
	
	//摄像头1,摄像头2图像输出
	wire fifo1_aclr;
	wire fifo1_wrreq;
	wire [15:0] fifo1_wrdata;
	wire fifo2_aclr;
	wire fifo2_wrreq;
	wire [15:0] fifo2_wrdata;
	DVP_Capture DVP_Capture1(
		.Rst_n(Init_Done),
		.PCLK(Camera1_pclk),
		.Vsync(Camera1_vsync),
		.Href(Camera1_href),
		.Data(Camera1_data),
		.ImageState(fifo1_aclr),
		.DataValid(fifo1_wrreq),
		.DataPixel(fifo1_wrdata),
		.Xaddr(),
		.Yaddr()
	),
	DVP_Capture2(
		.Rst_n(Init_Done),
		.PCLK(Camera2_pclk),
		.Vsync(Camera2_vsync),
		.Href(Camera2_href),
		.Data(Camera2_data),
		.ImageState(fifo2_aclr),
		.DataValid(fifo2_wrreq),
		.DataPixel(fifo2_wrdata),
		.Xaddr(),
		.Yaddr()
	);
	
	//摄像头1,摄像头2图像缓存
	wire cache_rdclk;
	wire cache1_rdreq;
	wire [15:0] cache1_rddata;
	wire [13:0] rdusedw1;
	wire cache2_rdreq;
	wire [15:0] cache2_rddata;
	wire [13:0] rdusedw2;
	cache cache1(
		.wrclk(Camera1_pclk),
		.aclr(fifo1_aclr),
		.wrreq(fifo1_wrreq),
		.wrdata(fifo1_wrdata),
		.rdclk(cache_rdclk),
		.rdreq(cache1_rdreq),
		.rddata(cache1_rddata),
		.rdusedw(rdusedw1)
	),
	cache2(
		.wrclk(Camera2_pclk),
		.aclr(fifo2_aclr),
		.wrreq(fifo2_wrreq),
		.wrdata(fifo2_wrdata),
		.rdclk(cache_rdclk),
		.rdreq(cache2_rdreq),
		.rddata(cache2_rddata),
		.rdusedw(rdusedw2)
	);
	
	wire fifo_wrreq;
	wire [15:0] fifo_wrdata;
	wire wrclk;
	controller
	#(
		.IMG_WIDTH(IMAGE_WIDTH)
	)
	u_controller(
		//.clk(clk_30m),
		.clk(Camera1_pclk),
		.rst_n(Rst_n),
		.rdclk(cache_rdclk),
		.rdreq1(cache1_rdreq),
		.rddata1(cache1_rddata),
		.rdusedw1(rdusedw1),
		.rdreq2(cache2_rdreq),
		.rddata2(cache2_rddata),
		.rdusedw2(rdusedw2),
		.wrclk(wrclk),
		.fifo_wrreq(fifo_wrreq),
		.fifo_wrdata(fifo_wrdata)
	);
	
	wire [15:0] RGB_DATA;
	wire DataReq;
	DDR2_Control_4Port I_DDR2_Control_4Port
	(
		//	HOST Side
		.REF_CLK			(Clk50M),
		.RESET_N			(Rst_n & sys_rst_n),
			  
		.GLOBAL_RES		(GLOBAL_RES),
		
		//	FIFO Write Side 1
		.WR1_DATA		(fifo_wrdata),
		.WR1				(fifo_wrreq),
		.WR1_ADDR		(0),
		.WR1_MAX_ADDR	(IMAGE_WIDTH * IMAGE_HEIGHT * 2),
		.WR1_LENGTH		(256),
		.WR1_LOAD		(!GLOBAL_RES),
		.WR1_CLK			(wrclk),
		
		//	FIFO Read Side 1
		.RD1_DATA		(RGB_DATA),
		.RD1				(DataReq),
		.RD1_ADDR		(0),
		.RD1_MAX_ADDR	(IMAGE_WIDTH * IMAGE_HEIGHT * 2),
		//.RD1_MAX_ADDR	(IMAGE_WIDTH * (IMAGE_HEIGHT + 360) * 2),
		.RD1_LENGTH		(256),
		.RD1_LOAD		(!GLOBAL_RES | frame_begin),	
		.RD1_CLK			(clk_vga),

		//	DDR Side
		.mem_odt			(DDR2_ODT),
		.mem_cs_n		(DDR2_CS),
		.mem_cke			(DDR2_CKE),
		.mem_addr		(DDR2_ADDR),
		.mem_ba			(DDR2_BA),
		.mem_ras_n		(DDR2_RAS),
		.mem_cas_n		(DDR2_CAS),
		.mem_we_n		(DDR2_WE),
		.mem_dm			(DDR2_DM),
		.mem_clk			(DDR2_CLK),
		.mem_clk_n		(DDR2_CLK_N),
		.mem_dq			(DDR2_DQ),
		.mem_dqs			(DDR2_DQS)
	);
	
	wire [4:0]  video_r;
	wire [5:0]  video_g;
	wire [4:0]  video_b;
	wire frame_begin;

	disp_driver disp_driver(
		.ClkDisp(clk_vga),
		.Rst_n(Rst_n),
		.Data(RGB_DATA),
		.DataReq(DataReq),
		.H_Addr(),
		.V_Addr(),
		.Disp_HS(vga_hs),
		.Disp_VS(vga_vs),
		.Disp_Red(video_r),
		.Disp_Green(video_g),
		.Disp_Blue(video_b),
		.frame_begin(frame_begin),
		.Disp_DE(vga_de),
		.Disp_PCLK(vga_clk)
	);
	
	//vga display
	assign vga_data = {video_r, 3'd0, video_g, 2'd0, video_b, 3'd0}; //vga数据输出

endmodule
